/* $Id$ */
#ifndef MSP_TIME_DATETIME_H_
#define MSP_TIME_DATETIME_H_

#include <string>
#include "types.h"

namespace Msp {
namespace Time {

class TimeDelta;
class TimeStamp;

/**
Provides handling of arbitary dates and times.  Can represent a moment of time
in the range of about ±2.1×10⁹ years.  It can also be formatted into a string
for presentation to the user.

Due to the complex internal representation, arithmetic operations on a DateTime
are relatively slow.  For purposes of internal scheduling in a program, a
TimeStamp is a better choice.
*/
class DateTime
{
public:
	DateTime(const TimeStamp &);
	DateTime(int, unsigned char, unsigned char);
	DateTime(int, unsigned char, unsigned char, unsigned char, unsigned char, unsigned char);
	DateTime(int, unsigned char, unsigned char, unsigned char, unsigned char, unsigned char, unsigned);
	
	int           get_year() const   { return year; }
	unsigned char get_month() const  { return month; }
	unsigned char get_mday() const   { return mday; }
	unsigned char get_hour() const   { return hour; }
	unsigned char get_minute() const { return minute; }
	unsigned char get_second() const { return second; }
	unsigned      get_usec() const   { return usec; }

	void add_days(int);

	DateTime operator+(const TimeDelta &) const;
	DateTime &operator+=(const TimeDelta &);

	bool operator==(const DateTime &d) const { return cmp(d)==0; }
	bool operator!=(const DateTime &d) const { return cmp(d)!=0; }
	bool operator<(const DateTime &d) const { return cmp(d)<0; }
	bool operator<=(const DateTime &d) const { return cmp(d)<=0; }
	bool operator>(const DateTime &d) const { return cmp(d)>0; }
	bool operator>=(const DateTime &d) const { return cmp(d)>=0; }

	int cmp(const DateTime &) const;
	TimeStamp get_timestamp() const;
	std::string format(const std::string &) const;
private:
	int           year;
	unsigned char month;
	unsigned char mday;
	unsigned char hour;
	unsigned char minute;
	unsigned char second;
	unsigned      usec;

	void add_raw(RawTime);
	void normalize();
	void validate() const;
};

} // namespace Time
} // namespace Msp

#endif
